const R = require('ramda');
const CONSTANTS = require('@src/utils/constants');

module.exports = {
    validateStage,
    validateLocale,
    validateUtterance,
    validateSkillId,
    validateNextToken,
    validateMaxResults,
    validateSortFieldForListModelVersions,
    validateSortDirectionForListModelVersions,
};

const localeRegex = /^[a-z]{2}-[A-Z]{2}$/;
const stagesWithoutCert = [CONSTANTS.SKILL.STAGE.DEVELOPMENT, CONSTANTS.SKILL.STAGE.LIVE];
const stages = [stagesWithoutCert, CONSTANTS.SKILL.STAGE.CERTIFICATION];
// Sets the field that the sorting is applied to. The only valid value is version.
const validSortFieldForListModelVersions = ["version"];
// Sets the sorting direction of the versions returned in the response. Valid values are asc and desc. 
// When set to asc the versions are returned in ascending order, and when set to desc the versions are returned in descending order.
const validSortDirectionForListModelVersions = ["asc", "desc"];

/**
 * General stage option validation logic
 * @param {String} stage
 * @param {Object} opts
 */
function validateStage(stage, opts = {}) {
    if (typeof opts.field === 'undefined') {
        opts.field = 'stage';
    }

    validateWrapper(stage, opts, () => {
        validateString(stage);
        const allowCert = typeof opts.allowCert === 'undefined' ? false : opts.allowCert;
        const validStages = allowCert ? stages : stagesWithoutCert;
        const valid = validStages.indexOf(stage) !== -1;
        if (!valid) {
            throw (`Value must be in (${validStages.join(',')}).`);
        }
    });
}

/**
 * General locale option validation logic
 * @param {String} locale
 * @param {Object} opts
 */
function validateLocale(locale, opts = {}) {
    if (typeof opts.field === 'undefined') {
        opts.field = 'locale';
    }

    validateWrapper(locale, opts, () => {
        validateString(locale);
        if (!localeRegex.test(locale)) {
            throw ('Invalid Format. Example: en-US.');
        }
    });
}

/**
 * General utterance option validation logic
 * @param {String} utterance
 * @param {Object} opts
 */
function validateUtterance(utterance, opts = {}) {
    if (typeof opts.field === 'undefined') {
        opts.field = 'utterance';
    }

    validateWrapper(utterance, opts, () => {
        validateString(utterance);
    });
}

/**
 * General skill-id option validation logic
 * @param {String} skillId
 * @param {Object} opts
 */
function validateSkillId(skillId, opts = {}) {
    if (typeof opts.field === 'undefined') {
        opts.field = 'skill-id';
    }

    validateWrapper(skillId, opts, () => {
        validateString(skillId);
    });
}

/**
 * General nextToken option validation logic
 * @param {String} nextToken
 * @param {Object} opts
 */
function validateNextToken(nextToken, opts = {}) {
    if (typeof opts.field === 'undefined') {
        opts.field = 'next-token';
    }

    validateWrapper(nextToken, opts, () => {
        validateString(nextToken);
    });
}

/**
 * General maxResults option validation logic
 * @param {String} maxResults
 * @param {Object} opts
 */
function validateMaxResults(maxResults, opts = {}) {
    if (typeof opts.field === 'undefined') {
        opts.field = 'max-results';
    }

    validateWrapper(maxResults, opts, () => {
        validateString(maxResults);
    });
}

/**
 * General sortField option validation logic
 * @param {String} sortField
 * @param {Object} opts
 */
function validateSortFieldForListModelVersions(sortField, opts = {}) {
    if (typeof opts.field === 'undefined') {
        opts.field = 'sort-field';
    }

    validateWrapper(sortField, opts, () => {
        validateString(sortField);

        const valid = validSortFieldForListModelVersions.indexOf(sortField) !== -1;
        if (!valid) {
            throw (`Value must be in (${validSortFieldForListModelVersions.join(',')}).`);
        }
    });
}

/**
 * General sortDirection option validation logic
 * @param {String} sortDirection
 * @param {Object} opts
 */
function validateSortDirectionForListModelVersions(sortDirection, opts = {}) {
    if (typeof opts.field === 'undefined') {
        opts.field = 'sort-direction';
    }

    validateWrapper(sortDirection, opts, () => {
        validateString(sortDirection);

        const valid = validSortDirectionForListModelVersions.indexOf(sortDirection) !== -1;
        if (!valid) {
            throw (`Value must be in (${validSortDirectionForListModelVersions.join(',')}).`);
        }
    });
}

function validateString(val) {
    if (!R.is(String, val)) {
        throw ('Must be a string.');
    }
    if (R.isEmpty(val.trim())) {
        throw ('Value must not be empty.');
    }
}

function validateWrapper(val, options = {}, callback) {
    const { field, required = true } = options;
    try {
        if (!R.isNil(val)) {
            if (callback) {
                callback();
            }
        } else if (required) {
            throw ('Field is required and must be set.');
        }
    } catch (err) {
        throw (`Please provide valid input for option: ${field}. ${err}`);
    }
}
